@@ -39,11 +39,12 @@ class DownstreamConnectionPool;
 class Worker;
 struct DownstreamAddrGroup;
 struct DownstreamAddr;
+struct DNSQuery;
 
 class HttpDownstreamConnection : public DownstreamConnection {
 public:
   HttpDownstreamConnection(const std::shared_ptr<DownstreamAddrGroup> &group,
-                           ssize_t initial_addr_idx, struct ev_loop *loop,
+                           DownstreamAddr *addr, struct ev_loop *loop,
                            Worker *worker);
   virtual ~HttpDownstreamConnection();
   virtual int attach_downstream(Downstream *downstream);
@@ -64,9 +65,13 @@ public:
 
   virtual bool poolable() const;
 
-  virtual DownstreamAddrGroup *get_downstream_addr_group() const;
+  virtual const std::shared_ptr<DownstreamAddrGroup> &
+  get_downstream_addr_group() const;
   virtual DownstreamAddr *get_addr() const;
 
+  int initiate_connection();
+
+  int write_first();
   int read_clear();
   int write_clear();
   int read_tls();
@@ -79,21 +84,38 @@ public:
   void signal_write();
   int actual_signal_write();
 
+  // Returns address used to connect to backend.  Could be nullptr.
+  const Address *get_raddr() const;
+
   int noop();
 
+  int process_blocked_request_buf();
+
 private:
   Connection conn_;
-  std::function<int(HttpDownstreamConnection &)> do_read_, do_write_,
-      do_signal_write_;
+  std::function<int(HttpDownstreamConnection &)> on_read_, on_write_,
+      signal_write_;
   Worker *worker_;
   // nullptr if TLS is not used.
   SSL_CTX *ssl_ctx_;
-  const std::shared_ptr<DownstreamAddrGroup> &group_;
+  std::shared_ptr<DownstreamAddrGroup> group_;
   // Address of remote endpoint
   DownstreamAddr *addr_;
+  // Actual remote address used to contact backend.  This is initially
+  // nullptr, and may point to either &addr_->addr, or
+  // resolved_addr_.get().
+  const Address *raddr_;
+  // Resolved IP address if dns parameter is used
+  std::unique_ptr<Address> resolved_addr_;
+  std::unique_ptr<DNSQuery> dns_query_;
   IOControl ioctrl_;
   http_parser response_htp_;
-  ssize_t initial_addr_idx_;
+  // true if first write succeeded.
+  bool first_write_done_;
+  // true if this object can be reused
+  bool reusable_;
+  // true if request header is written to request buffer.
+  bool request_header_written_;
 };
 
 } // namespace shrpx
